/**
 * @file Tropykus
 * @desc This file defines the constructor of the `Tropykus` class.
 * @hidden
 */
import { ethers } from "ethers";
import * as eth from "./eth";
import * as util from "./util";
import { Provider, CompoundOptions, CompoundInstance } from "./types";
/**
 * Creates an instance of the Tropykus.js SDK.
 *
 * @param {Provider | string} [provider] Optional Ethereum network provider.
 *     Defaults to Ethers.js fallback mainnet provider.
 * @param {object} [options] Optional provider options.
 *
 * @example
 *
 * ```
 * var tropykus = new Tropykus(window.ethereum); // web browser
 *
 * var tropykus = new Tropykus('http://127.0.0.1:8545'); // HTTP provider
 *
 * var tropykus = new Tropykus(); // Uses Ethers.js fallback mainnet (for testing only)
 *
 * var tropykus = new Tropykus('https://public-node.testnet.rsk.co'); // Uses Ethers.js fallback (for testing only)
 *
 * // Init with private key (server side)
 * var tropykus = new Tropykus('https://public-node.rsk.co', {
 *   privateKey: '0x_your_private_key_', // preferably with environment variable
 * });
 *
 * // Init with HD mnemonic (server side)
 * var tropykus = new Tropykus('https://public-node.rsk.co' {
 *   mnemonic: 'clutch captain shoe...', // preferably with environment variable
 * });
 * ```
 *
 * Compound III (Comet) Object Initialization. This accepts the same parameters
 *     as the `Compound` constructor. An error will be thrown initially and
 *     whenever a method is called if the provider does not match the network of
 *     the specific Comet deployment. The SDK constants as well as a method in
 *     the Comet documentation note the Comet deployments that Compound.js
 *     supports.
 *
 * ```
 * var compound = new Compound(window.ethereum);
 * var comet = compound.comet.MAINNET_USDC(); // provider from `compound` will be used unless on is explicitly passed
 * ```
 *
 * @returns {object} Returns an instance of the Tropykus.js SDK.
 */
declare const Tropykus: {
    (provider?: Provider | string, options?: CompoundOptions): CompoundInstance;
    eth: typeof eth;
    util: typeof util;
    _ethers: typeof ethers;
    decimals: {
        cAAVE: number;
        cBAT: number;
        cCOMP: number;
        cDAI: number;
        cETH: number;
        cFEI: number;
        cLINK: number;
        cMKR: number;
        cREP: number;
        cSAI: number;
        cSUSHI: number;
        cTUSD: number;
        cUNI: number;
        cUSDC: number;
        cUSDP: number;
        cUSDT: number;
        cWBTC: number;
        cYFI: number;
        cZRX: number;
        AAVE: number;
        BAT: number;
        BTC: number;
        COMP: number;
        DAI: number;
        ETH: number;
        FEI: number;
        LINK: number;
        MKR: number;
        REP: number;
        SAI: number;
        SUSHI: number;
        TUSD: number;
        UNI: number;
        USDC: number;
        USDP: number;
        USDT: number;
        WBTC: number;
        YFI: number;
        ZRX: number;
        DOC: number;
        RBTC: number;
        BPRO: number;
        USDRIF: number;
    };
};
export = Tropykus;
