"use strict";
/**
 * @file Tropykus
 * @desc This file defines the constructor of the `Tropykus` class.
 * @hidden
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var ethers_1 = require("ethers");
var eth = require("./eth");
var util = require("./util");
var comptroller = require("./comptroller");
var cToken = require("./cToken");
var priceFeed = require("./priceFeed");
var api = require("./api");
var constants_1 = require("./constants");
// Turn off Ethers.js warnings
ethers_1.ethers.utils.Logger.setLogLevel(ethers_1.ethers.utils.Logger.levels.ERROR);
/**
 * Creates an instance of the Tropykus.js SDK.
 *
 * @param {Provider | string} [provider] Optional Ethereum network provider.
 *     Defaults to Ethers.js fallback mainnet provider.
 * @param {object} [options] Optional provider options.
 *
 * @example
 *
 * ```
 * var tropykus = new Tropykus(window.ethereum); // web browser
 *
 * var tropykus = new Tropykus('http://127.0.0.1:8545'); // HTTP provider
 *
 * var tropykus = new Tropykus(); // Uses Ethers.js fallback mainnet (for testing only)
 *
 * var tropykus = new Tropykus('https://public-node.testnet.rsk.co'); // Uses Ethers.js fallback (for testing only)
 *
 * // Init with private key (server side)
 * var tropykus = new Tropykus('https://public-node.rsk.co', {
 *   privateKey: '0x_your_private_key_', // preferably with environment variable
 * });
 *
 * // Init with HD mnemonic (server side)
 * var tropykus = new Tropykus('https://public-node.rsk.co' {
 *   mnemonic: 'clutch captain shoe...', // preferably with environment variable
 * });
 * ```
 *
 * Compound III (Comet) Object Initialization. This accepts the same parameters
 *     as the `Compound` constructor. An error will be thrown initially and
 *     whenever a method is called if the provider does not match the network of
 *     the specific Comet deployment. The SDK constants as well as a method in
 *     the Comet documentation note the Comet deployments that Compound.js
 *     supports.
 *
 * ```
 * var compound = new Compound(window.ethereum);
 * var comet = compound.comet.MAINNET_USDC(); // provider from `compound` will be used unless on is explicitly passed
 * ```
 *
 * @returns {object} Returns an instance of the Tropykus.js SDK.
 */
var Tropykus = function (provider, options) {
    if (provider === void 0) { provider = "https://public-node.rsk.co"; }
    if (options === void 0) { options = {}; }
    var originalProvider = provider;
    options.provider = provider || options.provider;
    provider = eth._createProvider(options);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    var instance = __assign(__assign(__assign(__assign({ _originalProvider: originalProvider, _provider: provider }, comptroller), cToken), priceFeed), api);
    // Instance needs to know which network the provider connects to, so it can
    //     use the correct contract addresses.
    instance._networkPromise = eth
        .getProviderNetwork(provider)
        .then(function (network) {
        delete instance._networkPromise;
        instance._network = network;
    });
    return instance;
};
Tropykus.eth = eth;
Tropykus.util = util;
Tropykus._ethers = ethers_1.ethers;
Tropykus.decimals = constants_1.decimals;
Object.assign(Tropykus, constants_1.constants);
module.exports = Tropykus;
//# sourceMappingURL=index.js.map